<?php

namespace Drupal\domain_language;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Config\ConfigFactoryOverrideInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\StorageInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\domain_config\DomainConfigOverrider;

/**
 * Overrides configuration values based on the active domain and language.
 *
 * @package Drupal\domain_language
 */
class DomainLanguageOverrider implements ConfigFactoryOverrideInterface {

  /**
   * The domain context of the request.
   *
   * @var \Drupal\domain\DomainInterface
   */
  protected $domain;

  /**
   * The domain negotiator.
   *
   * @var \Drupal\domain\DomainNegotiatorInterface|null
   */
  protected $domainNegotiator;

  /**
   * Nested Level.
   *
   * @var int
   */
  protected $nestedLevel;

  /**
   * Constructs a DomainLanguageOverrider object.
   *
   * @param \Drupal\Core\Session\AccountProxyInterface $currentUser
   *   The current user.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The configuration factory.
   * @param \Drupal\domain_config\DomainConfigOverrider $overrider
   *   The domain config overrider service.
   */
  public function __construct(
    protected AccountProxyInterface $currentUser,
    protected ConfigFactoryInterface $configFactory,
    protected DomainConfigOverrider $overrider,
  ) {
    $this->nestedLevel = 0;
  }

  /**
   * {@inheritdoc}
   */
  public function loadOverrides($names) {
    global $config;

    $this->nestedLevel++;
    $overrides = [];

    if ($domain = $this->getDomainNegotiator()->getActiveDomain()) {
      foreach ($names as $name) {
        if ($name == 'system.site' && !$this->currentUser->hasPermission('bypass language restrictions')) {
          $configs = $this->overrider->loadOverrides([$name]);

          if (!empty($configs[$name])) {
            // Initialize site settings.
            $overrides[$name] = $configs[$name];
          }
        }
        elseif ($name == 'language.negotiation' && !$this->currentUser->hasPermission('bypass language restrictions')) {
          $languages = $this->configFactory->getEditable('domain.language.' . $domain->getOriginalId() . '.' . $name)->get(
            'languages'
          );

          // Setting array as null will reset keys.
          if (!empty($languages)) {
            // @todo use alter mechanism to properly remove array entries.
            // @see: https://www.drupal.org/node/2829242
            $overrides[$name]['url']['prefixes'] = NULL;
            $overrides[$name]['url']['domains'] = NULL;

            // Set global config var to use second override mechanisms.
            if ($this->nestedLevel == 1) {
              $negotiations = $this->configFactory->getEditable($name)->getRawData();

              // @todo use 'Drupal\Core\Site\Settings' when available.
              // @see: https://www.drupal.org/node/2183591
              $config[$name]['url']['prefixes'] = array_intersect_key(
                $negotiations['url']['prefixes'],
                $languages
              );
              $config[$name]['url']['domains'] = array_intersect_key(
                $negotiations['url']['domains'],
                $languages
              );
            }
          }
        }
      }
    }

    $this->nestedLevel--;

    return $overrides;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheSuffix() {
    // We use the request URL instead of the domain ID to avoid
    // early domain negotiation.
    $request = \Drupal::request();
    return $request->getSchemeAndHttpHost() . $request->getBaseUrl();
  }

  /**
   * {@inheritdoc}
   */
  public function createConfigObject($name, $collection = StorageInterface::DEFAULT_COLLECTION) {
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheableMetadata($name) {
    $metadata = new CacheableMetadata();
    $metadata->addCacheContexts(['url.site']);

    return $metadata;
  }

  /**
   * Retrieves the domain negotiator.
   *
   * @return \Drupal\domain\DomainNegotiatorInterface
   *   The domain negotiator.
   */
  protected function getDomainNegotiator() {
    if (!isset($this->domainNegotiator)) {
      $this->domainNegotiator = \Drupal::service('domain.negotiator');
    }
    return $this->domainNegotiator;
  }

}
