<?php

namespace Drupal\domain_language\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Link;
use Drupal\Core\Routing\RouteBuilderInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form to configure languages per domain.
 *
 * @package Drupal\domain_language\Form
 */
class DomainLanguageForm extends EntityForm {

  use StringTranslationTrait;

  const DEFAULT_LANGUAGE_SITE = '***LANGUAGE_site_default***';

  /**
   * The system site configuration.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $systemSiteConfig;

  /**
   * Constructs a new DomainLanguageForm object.
   *
   * @param \Drupal\Core\Language\LanguageManagerInterface $languageManager
   *   The language manager service.
   * @param \Drupal\Core\Routing\RouteBuilderInterface $routeBuilder
   *   The router builder service.
   */
  public function __construct(
    protected LanguageManagerInterface $languageManager,
    protected RouteBuilderInterface $routeBuilder,
  ) {
    $this->systemSiteConfig = $this->config('system.site');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('language_manager'),
      $container->get('router.builder')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form = parent::form($form, $form_state);
    // All available languages.
    $languages = $this->languageManager->getNativeLanguages();

    // Config without any override.
    $configRaw = $this->systemSiteConfig->getRawData();
    $defaultRaw = $configRaw['default_langcode'];

    // Load the domain default language.
    $config = $this->config('domain.config.' . $this->entity->getOriginalId() . '.system.site')->getRawData();
    $defaultLanguage = $config['default_langcode'] ?? self::DEFAULT_LANGUAGE_SITE;

    /** @var \Drupal\Core\Language\LanguageInterface $defaultLanguageRaw */
    $defaultLanguageRaw = $this->languageManager->getLanguage($defaultRaw);

    $options = [
      self::DEFAULT_LANGUAGE_SITE => $this->t(
        "Site's default language (@lang_name)",
        ['@lang_name' => $defaultLanguageRaw->getName()]
      ),
    ];
    foreach ($languages as $language) {
      $options[$language->getId()] = $language->getName();
    }

    $form['default_language'] = [
      '#type' => 'select',
      '#title' => $this->t('Default language'),
      '#options' => $options,
      '#description' => $this->t(
        'This will override the default language: %default.',
        ['%default' => $defaultLanguageRaw->getName()]
      ),
      '#default_value' => $defaultLanguage,
      '#required' => TRUE,
    ];

    $options = [];
    foreach ($languages as $language) {
      $options[$language->getId()] = $language->getName();
    }

    $config = $this->config('domain.language.' . $this->entity->getOriginalId() . '.language.negotiation');
    $data = $config->getRawData();

    $form['languages'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Languages allowed'),
      '#description' => $this->t('If none selected, all will be available. Default language will be added automatically.'),
      '#options' => $options,
      '#default_value' => $data['languages'] ?? [],
      '#required' => FALSE,
    ];

    $form['actions'] = [
      '#type' => 'container',
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#button_type' => 'primary',
      '#value' => $this->t('Save'),
    ];

    $form['actions']['cancel'] = Link::createFromRoute($this->t('Cancel'), 'domain.admin')->toRenderable();
    $form['actions']['cancel']['#attributes']['class'] = ['button', 'button--danger'];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $default_language = $form_state->getValue('default_language');
    $languages = array_filter($form_state->getValue('languages'));

    // Set default language into domain config file.
    $config = $this->configFactory()->getEditable('domain.config.' . $this->entity->getOriginalId() . '.system.site');
    if ($default_language == self::DEFAULT_LANGUAGE_SITE) {
      $data = $config->getRawData();
      unset($data['default_langcode']);
      $config->setData($data);

      if (empty($data)) {
        // Delete if config is now empty.
        $config->delete();
      }
      else {
        $config->save();
      }
    }
    else {
      $config->set('default_langcode', $default_language);
      $config->save();
    }

    // Set default language into domain language file.
    $config = $this->configFactory()->getEditable('domain.language.' . $this->entity->getOriginalId() . '.language.negotiation');
    if (empty($languages)) {
      $config->delete();
    }
    else {
      $languages[$default_language] = $default_language;
      $config->set('languages', $languages);
      $config->save();
    }

    // Remove any prefixes and domains in negotiation settings from
    // domain.config file to avoid any unexpected override.
    $config = $this->configFactory()->getEditable('domain.config.' . $this->entity->getOriginalId() . '.language.negotiation');
    $data = $config->getRawData();
    unset($data['url']['prefixes'], $data['url']['domains']);
    if (empty($data['url'])) {
      unset($data['url']);
    }
    if (empty($data)) {
      $config->delete();
    }
    else {
      $config->setData($data);
      $config->save();
    }

    $this->routeBuilder->rebuild();

    // Redirect to domain admin page.
    $form_state->setRedirect('domain.admin');
    $this->messenger()->addStatus('Domain language settings have been updated.');
  }

}
