<?php

namespace Drupal\Tests\domain_language\Functional;

use Drupal\Core\Url;
use Drupal\language\Entity\ConfigurableLanguage;
use Drupal\Tests\domain\Functional\DomainTestBase;

/**
 * Tests the domain language module.
 */
class DomainLanguageTest extends DomainTestBase {

  /**
   * Disable strict config schema checks.
   *
   * @var bool
   */
  protected $strictConfigSchema = FALSE;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'domain_language',
  ];

  /**
   * The domain storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $domainStorage;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->domainStorage = $this->container->get('entity_type.manager')->getStorage('domain');
    $this->languageManager = $this->container->get('language_manager');

    // Create five test domains.
    $this->domainCreateTestDomains(3);

    $this->drupalPlaceBlock('language_block:language_interface');

    // Add FR language.
    ConfigurableLanguage::createFromLangcode('fr')->save();
    ConfigurableLanguage::createFromLangcode('nl')->save();

    // Add path_prefix based language negotiation.
    $this->config('language.negotiation')
      ->set('url.source', 'path_prefix')
      ->set('url.prefixes', ['en' => 'en', 'fr' => 'fr', 'nl'])
      ->save();
  }

  /**
   * Tests the domain language functionality.
   */
  public function testDomainLanguageAccessForm(): void {
    $this->drupalLogin($this->rootUser);

    $this->drupalGet('admin/config/domain');
    // Check that the operations link is available.
    $this->clickLink('Languages', 1);
    $this->assertSession()->addressEquals('en/admin/config/domain/language/one_example_com');

    // Check that the task links are available.
    $this->clickLink('Edit');
    $this->assertSession()->addressEquals('en/admin/config/domain/edit/one_example_com');
    $this->findLink('Languages');
    $this->clickLink('Languages');
    $this->assertSession()->addressEquals('en/admin/config/domain/language/one_example_com');

    // Check default values for the domain language form.
    $this->assertSession()->fieldValueEquals('default_language', '***LANGUAGE_site_default***');
    $this->assertFalse($this->findField('languages[en]')->isChecked());
    $this->assertFalse($this->findField('languages[nl]')->isChecked());
    $this->assertFalse($this->findField('languages[fr]')->isChecked());

    // Test submitting a domain language form.
    $this->submitForm([
      'default_language' => 'nl',
      'languages[en]' => 'en',
      'languages[nl]' => 'nl',
    ], 'Save');

    $this->assertSession()->pageTextContains('Domain language settings have been updated.');
    $this->assertSession()->addressEquals('en/admin/config/domain');

    // Check default values when changed.
    $this->drupalGet('admin/config/domain/language/one_example_com');
    $this->assertSession()->fieldValueEquals('default_language', 'nl');
    $this->assertTrue($this->findField('languages[en]')->isChecked());
    $this->assertTrue($this->findField('languages[nl]')->isChecked());
    $this->assertFalse($this->findField('languages[fr]')->isChecked());

    $this->drupalGet('admin/config/domain/language/two_example_com');
    $this->submitForm([
      'default_language' => 'fr',
      'languages[en]' => 'en',
      'languages[fr]' => 'fr',
    ], 'Save');

    // Check that the domain languages are correctly saved in config.
    $domain_one_system_site_config = $this->config('domain.config.one_example_com.system.site');
    $this->assertEquals('nl', $domain_one_system_site_config->get('default_langcode'));
    $domain_one_domain_language = $this->config('domain.language.one_example_com.language.negotiation');
    $this->assertEquals(['en' => 'en', 'nl' => 'nl'], $domain_one_domain_language->get('languages'));

    $domain_two_system_site_config = $this->config('domain.config.two_example_com.system.site');
    $this->assertEquals('fr', $domain_two_system_site_config->get('default_langcode'));
    $domain_two_domain_language = $this->config('domain.language.two_example_com.language.negotiation');
    $this->assertEquals(['en' => 'en', 'fr' => 'fr'], $domain_two_domain_language->get('languages'));

    // Check that the default config is not changed.
    $system_site_config = $this->config('system.site');
    $this->assertEquals('en', $system_site_config->get('default_langcode'));

    // Check that the language switcher shows the right links.
    /** @var \Drupal\domain\DomainInterface $domain */
    $domain = $this->domainStorage->load('one_example_com');
    $this->drupalGet($domain->getUrl());
    $this->assertSession()->pageTextContains('English');
    $this->assertSession()->pageTextContains('Dutch');
    $this->assertSession()->pageTextNotContains('French');

    // Check that the language switcher shows the right links.
    /** @var \Drupal\domain\DomainInterface $domain */
    $domain = $this->domainStorage->load('two_example_com');
    $this->drupalGet($domain->getUrl());
    $this->assertSession()->pageTextContains('English');
    $this->assertSession()->pageTextContains('French');
    $this->assertSession()->pageTextNotContains('Dutch');
  }

}
